@@ -67,6 +67,8 @@
 
 #include <modest-text-utils.h>
 
+#define DISABLE_GET_UNREAD_MSGS_FOR_MULTI_MAILBOX 1
+
 typedef struct 
 {
 	gchar *to;
@@ -537,6 +539,7 @@ find_msg_async_cb (TnyFolder *folder,
 				ModestMailOperation *mail_op;
 				
                                 tny_header_set_flag (header, TNY_HEADER_FLAG_SEEN);
+				modest_platform_emit_msg_read_changed_signal (msg_uid, TRUE);
 				/* Sync folder, we need this to save the seen flag */
 				mail_op = modest_mail_operation_new (NULL);
 				modest_mail_operation_queue_add (modest_runtime_get_mail_operation_queue (),
@@ -663,7 +666,24 @@ on_idle_open_message_performer (gpointer user_data)
 	return FALSE;
 }
 
-static gint 
+static void
+on_open_message_msg_found (TnyFolder *folder,
+	gboolean cancelled, TnyMsg *msg, GError *err, gpointer user_data)
+{
+	OpenMsgPerformerInfo *info;
+
+	info = (OpenMsgPerformerInfo *)user_data;
+	if (msg) {
+		info->connect = FALSE;
+		g_object_unref (msg);
+	} else {
+		info->connect = TRUE;
+	}
+
+	g_idle_add (on_idle_open_message_performer, info);
+}
+
+static gint
 on_open_message (GArray * arguments, gpointer data, osso_rpc_t * retval)
 {
  	osso_rpc_t val;
@@ -721,20 +741,17 @@ on_open_message (GArray * arguments, gpointer data, osso_rpc_t * retval)
 			info->connect = FALSE;
 		}
 		if (folder) {
-			TnyMsg *msg = tny_folder_find_msg (folder, uri, NULL);
-			if (msg) {
-				info->connect = FALSE;
-				g_object_unref (msg);
-			} else {
-				info->connect = TRUE;
-			}
+			/*TnyMsg *msg = tny_folder_find_msg (folder, uri, NULL);*/
+			/* Get message */
+			tny_folder_find_msg_async (folder, uri, on_open_message_msg_found, NULL, info);
 			g_object_unref (folder);
 		}
-
-		/* We need to call it into an idle to get
-		   modest_platform_connect_and_perform into the main
-		   loop */
-		g_idle_add (on_idle_open_message_performer, info);
+		else {
+			/* We need to call it into an idle to get
+			   modest_platform_connect_and_perform into the main
+			   loop */
+			g_idle_add (on_idle_open_message_performer, info);
+		}
 		osso_retval = OSSO_OK;
 	} else {
 		g_free (uri);
@@ -983,8 +1000,6 @@ on_idle_send_receive(gpointer user_data)
 	return FALSE;
 }
 
-
-
 static gint 
 on_dbus_method_dump_send_queues (DBusConnection *con, DBusMessage *message)
 {
@@ -1255,6 +1270,41 @@ on_send_receive_full (GArray *arguments, gpointer data, osso_rpc_t * retval)
 	return OSSO_OK;
 }
 
+static gboolean
+on_idle_update_folder_counts (gpointer userdata)
+{
+	ModestMailOperation *mail_op;
+	gchar *account_id = (gchar *) userdata;
+
+	mail_op = modest_mail_operation_new (NULL);
+	modest_mail_operation_queue_add (modest_runtime_get_mail_operation_queue (),
+					 mail_op);
+	modest_mail_operation_update_folder_counts (mail_op, account_id);
+	g_object_unref (mail_op);
+	g_free (account_id);
+
+	return FALSE;
+
+}
+
+static gint
+on_update_folder_counts (GArray *arguments, gpointer data, osso_rpc_t * retval)
+{
+ 	osso_rpc_t val;
+	gchar *account_id;
+
+	val = g_array_index (arguments, osso_rpc_t, MODEST_DBUS_SEND_RECEIVE_FULL_ARG_ACCOUNT_ID);
+	account_id = g_strdup (val.value.s);
+
+	if (account_id != NULL) {
+		g_idle_add (on_idle_update_folder_counts, g_strdup (account_id));
+	}
+
+	g_free (account_id);
+
+	return OSSO_OK;
+}
+
 static gint
 on_open_default_inbox(GArray * arguments, gpointer data, osso_rpc_t * retval)
 {
@@ -1365,11 +1415,6 @@ on_idle_top_application (gpointer user_data)
 
 		mgr = modest_runtime_get_window_mgr ();
 		window = (GtkWidget *) modest_window_mgr_show_initial_window (mgr);
-		if (window) {
-			modest_platform_remove_new_mail_notifications (FALSE);
-		} else {
-			g_printerr ("modest: failed to get main window instance\n");
-		}
 	}
 
 	gdk_threads_leave (); /* CHECKED */
@@ -1500,13 +1545,41 @@ on_idle_show_memory_low (gpointer user_data)
 static gboolean
 on_idle_present_modal (gpointer user_data)
 {
+	GtkWindow *current, *transient;
 	gdk_threads_enter ();
-	gtk_window_present (user_data);
+	current = (GtkWindow *) user_data;
+	while (GTK_IS_DIALOG (current)) {
+		transient = gtk_window_get_transient_for (GTK_WINDOW (current));
+		if (transient == NULL)
+			break;
+		else
+			current = transient;
+	}
+	gtk_window_present (current);
 	gdk_threads_leave ();
 
 	return FALSE;
 }
 
+/**
+  * This function checks if a modal dialog should be activated
+  * instead of other UI components on a DBus request, and if yes, starts the request
+  * @return TRUE if a modal dialog is about to be acitvated
+  * @note This function should be used before activating any Modest UI element on DBus request
+  */
+static gboolean
+modest_dbus_check_present_modal ()
+{
+	GtkWindow *dialog;
+
+	/* Check if there is already a dialog or note open */
+	dialog = modest_window_mgr_get_modal (modest_runtime_get_window_mgr());
+	if (dialog) {
+		g_idle_add (on_idle_present_modal, dialog);
+	}
+
+	return (NULL != dialog);
+}
 
 /* Callback for normal D-BUS messages */
 gint
@@ -1514,68 +1587,56 @@ modest_dbus_req_handler(const gchar * interface, const gchar * method,
 			GArray * arguments, gpointer data,
 			osso_rpc_t * retval)
 {
-	GtkWindow *dialog;
-
 	/* Check memory low conditions */
 	if (modest_platform_check_memory_low (NULL, FALSE)) {
 		g_idle_add (on_idle_show_memory_low, NULL);
 		goto param_error;
 	}
 
-	/* Check if there is already a dialog or note open */
-	dialog = modest_window_mgr_get_modal (modest_runtime_get_window_mgr());
-	if (dialog) {
-		g_idle_add (on_idle_present_modal, dialog);
-		return OSSO_OK;
-	}
-
 	if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_MAIL_TO) == 0) {
 		if (arguments->len != MODEST_DBUS_MAIL_TO_ARGS_COUNT)
 			goto param_error;
-		modest_runtime_set_allow_shutdown (TRUE);
-		return on_mail_to (arguments, data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_mail_to (arguments, data, retval);
 	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_OPEN_MESSAGE) == 0) {
 		if (arguments->len != MODEST_DBUS_OPEN_MESSAGE_ARGS_COUNT)
 			goto param_error;
-		modest_runtime_set_allow_shutdown (TRUE);
-		return on_open_message (arguments, data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_open_message (arguments, data, retval);
 	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_OPEN_ACCOUNT) == 0) {
 		if (arguments->len != MODEST_DBUS_OPEN_ACCOUNT_ARGS_COUNT)
 			goto param_error;
-		modest_runtime_set_allow_shutdown (TRUE);
-		return on_open_account (arguments, data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_open_account (arguments, data, retval);
 	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_SEND_RECEIVE) == 0) {
 		if (arguments->len != 0)
 			goto param_error;
-		return on_send_receive (arguments, data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_send_receive (arguments, data, retval);
 	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_SEND_RECEIVE_FULL) == 0) {
 		if (arguments->len != MODEST_DBUS_SEND_RECEIVE_FULL_ARGS_COUNT)
 			goto param_error;
-		return on_send_receive_full (arguments, data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_send_receive_full (arguments, data, retval);
+	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_UPDATE_FOLDER_COUNTS) == 0) {
+		if (arguments->len != MODEST_DBUS_UPDATE_FOLDER_COUNTS_ARGS_COUNT)
+			goto param_error;
+		return on_update_folder_counts (arguments, data, retval);
 	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_COMPOSE_MAIL) == 0) {
 		if (arguments->len != MODEST_DBUS_COMPOSE_MAIL_ARGS_COUNT)
 			goto param_error;
-		modest_runtime_set_allow_shutdown (TRUE);
-		return on_compose_mail (arguments, data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_compose_mail (arguments, data, retval);
 	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_DELETE_MESSAGE) == 0) {
 		if (arguments->len != MODEST_DBUS_DELETE_MESSAGE_ARGS_COUNT)
 			goto param_error;
-		return on_delete_message (arguments,data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_delete_message (arguments,data, retval);
 	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_OPEN_DEFAULT_INBOX) == 0) {
 		if (arguments->len != 0)
 			goto param_error;
-		modest_runtime_set_allow_shutdown (TRUE);
-		return on_open_default_inbox (arguments, data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_open_default_inbox (arguments, data, retval);
 	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_TOP_APPLICATION) == 0) {
 		if (arguments->len != 0)
 			goto param_error;
-		modest_runtime_set_allow_shutdown (TRUE);
-		return on_top_application (arguments, data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_top_application (arguments, data, retval);
 	} else if (g_ascii_strcasecmp (method, MODEST_DBUS_METHOD_OPEN_EDIT_ACCOUNTS_DIALOG) == 0) {
 		if (arguments->len != 0)
 			goto param_error;
-		modest_runtime_set_allow_shutdown (TRUE);
-		return on_open_edit_accounts_dialog (arguments, data, retval);
+		return modest_dbus_check_present_modal () ? OSSO_OK : on_open_edit_accounts_dialog (arguments, data, retval);
 	} else {
 		/* We need to return INVALID here so
 		 * libosso will return DBUS_HANDLER_RESULT_NOT_YET_HANDLED,
@@ -1604,6 +1665,23 @@ modest_dbus_req_handler(const gchar * interface, const gchar * method,
 	DBUS_TYPE_INT64_AS_STRING /* timestamp */ \
 	DBUS_STRUCT_END_CHAR_AS_STRING
 
+#define ACCOUNT_HIT_DBUS_TYPE \
+	DBUS_STRUCT_BEGIN_CHAR_AS_STRING \
+	DBUS_TYPE_INT64_AS_STRING /* timestamp */ \
+	DBUS_TYPE_STRING_AS_STRING /* subject */ \
+	DBUS_STRUCT_END_CHAR_AS_STRING
+
+
+#define ACCOUNTS_HIT_DBUS_TYPE \
+	DBUS_STRUCT_BEGIN_CHAR_AS_STRING \
+	DBUS_TYPE_STRING_AS_STRING \
+	DBUS_TYPE_STRING_AS_STRING \
+	DBUS_TYPE_STRING_AS_STRING \
+	DBUS_TYPE_INT64_AS_STRING \
+	DBUS_TYPE_ARRAY_AS_STRING \
+	ACCOUNT_HIT_DBUS_TYPE \
+	DBUS_STRUCT_END_CHAR_AS_STRING
+
 static DBusMessage *
 search_result_to_message (DBusMessage *reply,
 			   GList       *hits)
@@ -1836,6 +1914,605 @@ on_dbus_method_search (DBusConnection *con, DBusMessage *message)
 	modest_search_all_accounts (search, search_all_cb, helper);
 }
 
+static gint
+headers_cmp (TnyHeader *a, TnyHeader *b)
+{
+	time_t date_a, date_b;
+	date_a = tny_header_get_date_received (a);
+	date_b = tny_header_get_date_received (b);
+
+	return date_a - date_b;
+}
+
+/**
+  * The helper structure for handling GetUnreadMessages DBus request
+  */
+typedef struct {
+	TnyList *accounts_list;
+	TnyList *inboxes_list;
+	gint unread_msgs_count;
+	DBusConnection *con;
+	DBusMessage *message;
+	GList *account_hits_list;
+	ModestMailOperation *mail_op;
+	guint folder_requests_total; /**< Total get-folder requests number for multi-mailbox accounts */
+	guint folder_requests_done; /**< Done get-folder requests number for multi-mailbox accounts */
+	guint get_headers_tries_left;
+	TnyFolder *tmp_folder_ptr;
+} GetUnreadMessagesHelper;
+
+typedef struct {
+	gchar *account_id;
+	gchar *account_name;
+	gchar *store_protocol;
+	gint unread_count;
+	GList *header_list;
+} AccountHits;
+
+static void return_results (GetUnreadMessagesHelper *helper)
+{
+	DBusMessage *reply;
+
+	reply = dbus_message_new_method_return (helper->message);
+	if (reply) {
+		dbus_uint32_t serial = 0;
+		GList *node;
+		DBusMessageIter iter;
+		DBusMessageIter array_iter;
+
+		dbus_message_iter_init_append (reply, &iter);
+		dbus_message_iter_open_container (&iter,
+						  DBUS_TYPE_ARRAY,
+						  ACCOUNTS_HIT_DBUS_TYPE,
+						  &array_iter);
+		for (node = helper->account_hits_list; node != NULL; node = g_list_next (node)) {
+			AccountHits *ah = (AccountHits *) node->data;
+			const char *account_id;
+			const char *account_name;
+			const char *store_protocol;
+			gint64 unread_count;
+			DBusMessageIter ah_struct_iter;
+			DBusMessageIter sh_array_iter;
+			GList *result_node;
+
+			dbus_message_iter_open_container (&array_iter,
+							  DBUS_TYPE_STRUCT,
+							  NULL,
+							  &ah_struct_iter);
+			account_id = ah->account_id;
+			account_name = ah->account_name;
+			store_protocol = ah->store_protocol;
+			unread_count = ah->unread_count;
+			dbus_message_iter_append_basic (&ah_struct_iter,
+							DBUS_TYPE_STRING,
+							&account_id);
+			dbus_message_iter_append_basic (&ah_struct_iter,
+							DBUS_TYPE_STRING,
+							&account_name);
+			dbus_message_iter_append_basic (&ah_struct_iter,
+							DBUS_TYPE_STRING,
+							&store_protocol);
+			dbus_message_iter_append_basic (&ah_struct_iter,
+							DBUS_TYPE_INT64,
+							&unread_count);
+
+			dbus_message_iter_open_container (&ah_struct_iter,
+							  DBUS_TYPE_ARRAY,
+							  ACCOUNT_HIT_DBUS_TYPE,
+							  &sh_array_iter);
+			for (result_node = ah->header_list; result_node != NULL; result_node = g_list_next (result_node)) {
+				TnyHeader *header = (TnyHeader *) result_node->data;
+				DBusMessageIter sh_struct_iter;
+				gint64 ts = tny_header_get_date_received (header);
+				gchar *subject = tny_header_dup_subject (header);
+
+				dbus_message_iter_open_container (&sh_array_iter,
+								  DBUS_TYPE_STRUCT,
+								  NULL,
+								  &sh_struct_iter);
+				dbus_message_iter_append_basic (&sh_struct_iter,
+								DBUS_TYPE_INT64,
+								&ts);
+				dbus_message_iter_append_basic (&sh_struct_iter,
+								DBUS_TYPE_STRING,
+								(const gchar **) &subject); 
+
+				dbus_message_iter_close_container (&sh_array_iter,
+							   &sh_struct_iter); 
+				g_object_unref (header);
+			}
+			dbus_message_iter_close_container (&ah_struct_iter,
+							   &sh_array_iter); 
+
+			dbus_message_iter_close_container (&array_iter,
+							   &ah_struct_iter); 
+			g_free (ah->account_id);
+			g_free (ah->account_name);
+			g_free (ah->store_protocol);
+			g_list_free (ah->header_list);
+		}
+
+		dbus_message_iter_close_container (&iter,
+						   &array_iter); 
+		dbus_connection_send (helper->con, reply, &serial);
+		dbus_connection_flush (helper->con);
+		dbus_message_unref (reply);
+
+	}
+	g_list_free (helper->account_hits_list);
+	dbus_message_unref (helper->message);
+	g_object_unref (helper->accounts_list);
+	modest_mail_operation_queue_remove (modest_runtime_get_mail_operation_queue (),
+					    helper->mail_op);
+	g_object_unref (helper->mail_op);
+	g_slice_free (GetUnreadMessagesHelper, helper);
+}
+
+static void get_unread_messages_get_account (GetUnreadMessagesHelper *helper);
+static void get_unread_messages_get_headers (GetUnreadMessagesHelper *helper);
+static void get_unread_messages_get_headers_cb (TnyFolder *self, gboolean cancelled,
+	TnyList *headers, GError *err, GetUnreadMessagesHelper *helper);
+
+static gboolean
+send_get_headers_request (GetUnreadMessagesHelper *helper)
+{
+	TnyList *headers_list;
+
+	headers_list = TNY_LIST (tny_simple_list_new ());
+	tny_folder_get_headers_async (helper->tmp_folder_ptr, headers_list, FALSE,
+		(TnyGetHeadersCallback) get_unread_messages_get_headers_cb, NULL, helper);
+	g_object_unref (headers_list);
+	g_object_unref (helper->tmp_folder_ptr);
+	helper->tmp_folder_ptr = NULL;
+
+	return FALSE;
+}
+
+
+static void get_unread_messages_get_headers_cb (TnyFolder *self,
+						gboolean cancelled,
+						TnyList *headers,
+						GError *err,
+						GetUnreadMessagesHelper *helper)
+{
+	TnyIterator *acc_iterator;
+	TnyAccount *account;
+	TnyIterator *headers_iterator;
+	GList *result_list = NULL;
+	gint members_count = 0;
+	AccountHits *account_hits;
+	ModestProtocolType store_protocol_type;
+	ModestProtocol *store_protocol;
+	gint unread_count;
+	ModestProtocolRegistry *registry;
+
+	acc_iterator = tny_list_create_iterator (helper->accounts_list);
+	account = TNY_ACCOUNT (tny_iterator_get_current (acc_iterator));
+
+	headers_iterator = tny_list_create_iterator (headers);
+	unread_count = 0;
+	while (!tny_iterator_is_done (headers_iterator)) {
+		TnyHeader *header;
+		TnyHeaderFlags flags;
+
+		header = TNY_HEADER (tny_iterator_get_current (headers_iterator));
+		flags = tny_header_get_flags (header);
+		if (!(flags & TNY_HEADER_FLAG_SEEN)) {
+			unread_count++;
+			result_list = g_list_insert_sorted (result_list, g_object_ref (header), (GCompareFunc) headers_cmp);
+			if (members_count == helper->unread_msgs_count) {
+				g_object_unref (result_list->data);
+				result_list = g_list_delete_link (result_list, result_list);
+			} else {
+				members_count++;
+			}
+		}
+
+		g_object_unref (header);
+		tny_iterator_next (headers_iterator);
+	}
+	g_object_unref (headers_iterator);
+
+	registry = modest_runtime_get_protocol_registry ();
+	store_protocol_type = modest_tny_account_get_protocol_type (account);
+
+	/* Get the number of unread messages for plug-in based accounts */
+	if (modest_protocol_registry_protocol_type_is_provider (registry, store_protocol_type)) {
+		guint folder_unread_count;
+
+		folder_unread_count = tny_folder_get_unread_count (self);
+		if (folder_unread_count != unread_count &&
+			unread_count != helper->unread_msgs_count) {
+			/* the number of unread messages is incorrect, try again */
+			if (helper->get_headers_tries_left--) {
+				/* try again at a small timeout */
+				helper->tmp_folder_ptr = g_object_ref (self);
+
+				g_timeout_add (200, (GSourceFunc)send_get_headers_request, helper);
+
+				g_warning ("Getting unread messages, tries left: (%d)",
+					helper->get_headers_tries_left);
+				return;
+			}
+			else {
+				/* too many tries, give up for now */
+				unread_count = 0;
+				g_warning ("Getting unread messages: failed");
+			}
+		}
+		else {
+			unread_count = folder_unread_count;
+		}
+	}
+
+	account_hits = g_slice_new (AccountHits);
+	account_hits->account_id = g_strdup (modest_tny_account_get_parent_modest_account_name_for_server_account (account));
+	account_hits->account_name = g_strdup (tny_account_get_name (account));
+	store_protocol = modest_protocol_registry_get_protocol_by_type (registry, store_protocol_type);
+	account_hits->store_protocol = g_strdup (modest_protocol_get_name (store_protocol));
+	account_hits->header_list = result_list;
+	account_hits->unread_count = unread_count;
+
+	helper->account_hits_list = g_list_prepend (helper->account_hits_list, account_hits);
+
+	get_unread_messages_get_headers (helper);
+
+}
+
+static void
+get_unread_messages_get_headers (GetUnreadMessagesHelper *helper)
+{
+	TnyIterator *iterator;
+
+	iterator = tny_list_create_iterator (helper->inboxes_list);
+	do {
+		if (tny_iterator_is_done (iterator)) {
+			TnyIterator *accounts_iter;
+			TnyAccount *account;
+			g_object_unref (helper->inboxes_list);
+			helper->inboxes_list = NULL;
+
+			accounts_iter = tny_list_create_iterator (helper->accounts_list);
+			account = TNY_ACCOUNT (tny_iterator_get_current (accounts_iter));
+			g_object_unref (accounts_iter);
+
+			tny_list_remove (helper->accounts_list, (GObject *) account);
+			g_object_unref (account);
+			get_unread_messages_get_account (helper);
+			break;
+		} else {
+			TnyFolder *folder;
+
+			folder = TNY_FOLDER (tny_iterator_get_current (iterator));
+			if (folder) {
+				TnyList *headers_list;
+
+				headers_list = TNY_LIST (tny_simple_list_new ());
+				helper->get_headers_tries_left = 20;
+				tny_folder_get_headers_async (folder, headers_list, FALSE,
+					(TnyGetHeadersCallback) get_unread_messages_get_headers_cb,
+					NULL, helper);
+				g_object_unref (headers_list);
+				tny_list_remove (helper->inboxes_list, G_OBJECT (folder));
+				g_object_unref (folder);
+				break;
+			}
+		}
+
+		/* retrieved incorrect folder,
+		   move on to the next item in the inboxes_list */
+		tny_iterator_next (iterator);
+	} while (TRUE);
+
+	g_object_unref (iterator);
+}
+
+static void get_account_folders_cb (TnyFolderStore *self, gboolean cancelled, TnyList *list, GError *err, gpointer user_data)
+{
+	GetUnreadMessagesHelper *helper = (GetUnreadMessagesHelper *) user_data;
+	TnyIterator *iterator;
+	gboolean inbox_exists;
+
+	inbox_exists = FALSE;
+	iterator = tny_list_create_iterator (list);
+	while (!tny_iterator_is_done (iterator)) {
+		TnyFolder *folder;
+
+		folder = TNY_FOLDER (tny_iterator_get_current (iterator));
+		if (tny_folder_get_folder_type (folder) == TNY_FOLDER_TYPE_INBOX) {
+			tny_list_prepend (helper->inboxes_list, G_OBJECT (folder));
+			g_object_unref (folder);
+			inbox_exists = TRUE;
+			break;
+		}
+		g_object_unref (folder);
+		tny_iterator_next (iterator);
+	}
+	g_object_unref (iterator);
+
+	if (!inbox_exists) {
+		TnyAccount *account;
+		TnyIterator *accounts_it;
+		AccountHits *account_hit;
+		ModestProtocol *store_protocol;
+		ModestProtocolType store_protocol_type;
+
+		/* we have finished with the current account, prepare the account hit */
+		accounts_it = tny_list_create_iterator (helper->accounts_list);
+		account = TNY_ACCOUNT (tny_iterator_get_current (accounts_it));
+
+		if (!modest_tny_account_is_virtual_local_folders (account) &&
+			!modest_tny_account_is_memory_card_account (account)) {
+			/* the account does not have (yet) the inbox */
+			account_hit = g_slice_new (AccountHits);
+			account_hit->account_id = g_strdup (modest_tny_account_get_parent_modest_account_name_for_server_account (account));
+			account_hit->account_name = g_strdup (tny_account_get_name (account));
+			store_protocol_type = modest_tny_account_get_protocol_type (account);
+			store_protocol = modest_protocol_registry_get_protocol_by_type (modest_runtime_get_protocol_registry (), store_protocol_type);
+			account_hit->store_protocol = g_strdup (modest_protocol_get_name (store_protocol));
+			account_hit->header_list = NULL;
+			account_hit->unread_count = 0;
+
+			helper->account_hits_list = g_list_prepend (helper->account_hits_list, account_hit);
+		}
+	}
+
+	get_unread_messages_get_headers (helper);
+}
+
+static void get_multi_mailbox_account_folders_cb (TnyFolderStore *self, gboolean cancelled, TnyList *list, GError *err, gpointer user_data)
+{
+	GetUnreadMessagesHelper *helper = (GetUnreadMessagesHelper *) user_data;
+	TnyIterator *iterator;
+
+	/* another request has been finished */
+	helper->folder_requests_done += 1;
+
+	/* analize the folders we got */
+	iterator = tny_list_create_iterator (list);
+	while (!tny_iterator_is_done (iterator)) {
+		TnyFolder *folder;
+
+		folder = TNY_FOLDER (tny_iterator_get_current (iterator));
+		if (tny_folder_get_folder_type (folder) == TNY_FOLDER_TYPE_INBOX) {
+			const gchar *folder_id;
+
+			/**
+			 * @note for some reason, NMS account sometimes doubles the number of folder,
+			 *       temporary fix is to check if we already have a folder
+			 *       with the same id in the array before adding it to the result
+			 * @todo check why NMS account returns multiple number of the same folder
+			 */
+			folder_id = tny_folder_get_id (folder);
+			if (folder_id) {
+				gboolean alread_in_list;
+				TnyIterator *inboxes_it;
+
+				alread_in_list = FALSE;
+				inboxes_it = tny_list_create_iterator (helper->inboxes_list);
+				while (!tny_iterator_is_done (inboxes_it)) {
+					TnyFolder *folder;
+
+					folder = TNY_FOLDER (tny_iterator_get_current (inboxes_it));
+					if (folder) {
+						const gchar *inbox_id;
+
+						inbox_id = tny_folder_get_id (folder);
+						if (0 == g_strcmp0 (inbox_id, folder_id)) {
+							alread_in_list = TRUE;
+							break;
+						}
+					}
+
+					/* check the next folder in inboxes_list */
+					tny_iterator_next (inboxes_it);
+				}
+				g_object_unref (inboxes_it);
+
+				if (!alread_in_list) {
+					tny_list_prepend (helper->inboxes_list, G_OBJECT (folder));
+				}
+			}
+		}
+
+		/* Try to check if we have inbox sub-folders in this folder */
+		if (TNY_IS_FOLDER_STORE (folder)) {
+			TnyFolderStore *folder_store;
+
+			folder_store = TNY_FOLDER_STORE (folder);
+			if (folder_store) {
+				TnyList *folders_list;
+
+				helper->folder_requests_total += 1;
+				folders_list = tny_simple_list_new ();
+				tny_folder_store_get_folders_async (folder_store, folders_list,
+					NULL, FALSE, get_multi_mailbox_account_folders_cb, NULL, helper);
+				g_object_unref (folders_list);
+			}
+		}
+		g_object_unref (folder);
+		tny_iterator_next (iterator);
+	}
+	g_object_unref (iterator);
+
+	/* Check if we have handled all the inbox folders for multi-mailbox accounts */
+	if (helper->folder_requests_done == helper->folder_requests_total) {
+		TnyIterator *inboxes_it;
+		TnyIterator *accounts_it;
+		TnyAccount *account;
+		guint unread_messages;
+		AccountHits* account_hits;
+		ModestProtocol *store_protocol;
+		ModestProtocolType store_protocol_type;
+
+		/* Store the number of unread messages for the handled account */
+		unread_messages = 0;
+		accounts_it = tny_list_create_iterator (helper->accounts_list);
+		account = TNY_ACCOUNT (tny_iterator_get_current (accounts_it));
+		g_object_unref (accounts_it);
+		inboxes_it = tny_list_create_iterator (helper->inboxes_list);
+		while (!tny_iterator_is_done (inboxes_it)) {
+			TnyFolder *folder;
+
+			folder = TNY_FOLDER (tny_iterator_get_current (inboxes_it));
+			if (folder) {
+				TnyList *headers;
+
+				/* before the number of unread messages
+				   can be retrieved, we need to get folders */
+				headers = tny_simple_list_new ();
+				tny_folder_get_headers (folder, headers, FALSE, NULL);
+				g_object_unref (headers);
+				unread_messages += tny_folder_get_unread_count (folder);
+			}
+			tny_iterator_next (inboxes_it);
+		}
+		g_object_unref (inboxes_it);
+		g_object_unref (helper->inboxes_list);
+		helper->inboxes_list = NULL;
+		account_hits = g_slice_new (AccountHits);
+		account_hits->account_id = g_strdup (
+			modest_tny_account_get_parent_modest_account_name_for_server_account (account));
+		account_hits->account_name = g_strdup (tny_account_get_name (account));
+		store_protocol_type = modest_tny_account_get_protocol_type (account);
+		store_protocol = modest_protocol_registry_get_protocol_by_type (
+			modest_runtime_get_protocol_registry (), store_protocol_type);
+		account_hits->store_protocol = g_strdup (modest_protocol_get_name (store_protocol));
+		account_hits->header_list = NULL;
+		account_hits->unread_count = unread_messages;
+		helper->account_hits_list = g_list_prepend (helper->account_hits_list, account_hits);
+
+		/* remove the handled account from the accounts_list */
+		tny_list_remove (helper->accounts_list, G_OBJECT (account));
+		g_object_unref (account);
+
+		get_unread_messages_get_account (helper);
+	}
+}
+
+static void
+get_unread_messages_get_account (GetUnreadMessagesHelper *helper)
+{
+	TnyIterator *iterator;
+
+	/* Search through all accounts */
+	iterator = tny_list_create_iterator (helper->accounts_list);
+	do {
+		if (tny_iterator_is_done (iterator)) {
+			/* all results, then finish */
+			return_results (helper);
+			break;
+		} else {
+			TnyAccount *account;
+
+			account = TNY_ACCOUNT (tny_iterator_get_current (iterator));
+			if (account && TNY_FOLDER_STORE (account) && TNY_IS_FOLDER_STORE (TNY_FOLDER_STORE (account))) {
+				TnyList *folders_list;
+				ModestProtocolType protocol_type;
+
+				protocol_type = modest_tny_account_get_protocol_type (account);
+
+				folders_list = tny_simple_list_new ();
+				helper->inboxes_list =  tny_simple_list_new ();
+				if (MODEST_PROTOCOL_REGISTRY_TYPE_INVALID != protocol_type &&
+					modest_protocol_registry_protocol_type_has_tag (modest_runtime_get_protocol_registry (),
+						protocol_type, MODEST_PROTOCOL_REGISTRY_MULTI_MAILBOX_PROVIDER_PROTOCOLS)) {
+					/* For multi-mailbox protocols we only can get the number of unread messages,
+					   we will not even try to get their email headers */
+					helper->folder_requests_done = 0;
+					helper->folder_requests_total = 1;
+					tny_folder_store_get_folders_async (TNY_FOLDER_STORE (account),
+						folders_list, NULL, FALSE, get_multi_mailbox_account_folders_cb, NULL, helper);
+				}
+				else {
+					/* For non-multi-mailbox protocols we will get their email headers */
+					tny_folder_store_get_folders_async (TNY_FOLDER_STORE (account),
+						folders_list, NULL, FALSE, get_account_folders_cb, NULL, helper);
+				}
+				g_object_unref (folders_list);
+				g_object_unref (account);
+				break;
+			}
+		}
+		/* incorrect item detected, move on to the next item */
+		tny_iterator_next (iterator);
+	} while (TRUE);
+	g_object_unref (iterator);
+}
+
+/**
+ * This functions checkes if the item refers to a Modest virtual local folder
+ * @param[in] list The pointer to the list of accounts
+ * @param[in] item The current pointer in the list
+ * @param[in] match_data NULL pointer
+ * @return TRUE if the item is a Modest virtual local folder, FALSE otherwize
+ * @note This functions is used for removing local accounts
+ *       while collecting statistics information for unread messages
+ */
+static gboolean
+modest_local_accounts_matcher (TnyList *list G_GNUC_UNUSED, GObject *item, gpointer match_data G_GNUC_UNUSED)
+{
+	/* remove incorrect pointers */
+	g_return_val_if_fail (item, TRUE);
+	g_return_val_if_fail (TNY_ACCOUNT (item), TRUE);
+	g_return_val_if_fail (TNY_FOLDER_STORE (item), TRUE);
+	g_return_val_if_fail (TNY_IS_ACCOUNT (TNY_ACCOUNT (item)), TRUE);
+	g_return_val_if_fail (TNY_IS_FOLDER_STORE (TNY_FOLDER_STORE (item)), TRUE);
+
+	/* check if this is a virtual local folder */
+	return modest_tny_account_is_virtual_local_folders (TNY_ACCOUNT (item)) ||
+		modest_tny_account_is_memory_card_account (TNY_ACCOUNT (item));
+}
+
+static gboolean
+on_idle_get_unread_messages (GetUnreadMessagesHelper *helper)
+{
+	ModestTnyAccountStore *astore;
+
+	astore = modest_runtime_get_account_store ();
+
+	tny_account_store_get_accounts (TNY_ACCOUNT_STORE (astore),
+					helper->accounts_list,
+					TNY_ACCOUNT_STORE_STORE_ACCOUNTS);
+
+	/* remove the local accounts */
+	tny_list_remove_matches (helper->accounts_list, modest_local_accounts_matcher, NULL);
+
+	get_unread_messages_get_account (helper);
+
+	return FALSE;
+}
+
+static void
+on_dbus_method_get_unread_messages (DBusConnection *con, DBusMessage *message)
+{
+	dbus_bool_t  res;
+	dbus_int64_t unread_msgs_count_v;
+	DBusError error;
+	GetUnreadMessagesHelper *helper;
+
+	dbus_error_init (&error);
+
+	res = dbus_message_get_args (message,
+				     &error,
+				     DBUS_TYPE_INT32, &unread_msgs_count_v,
+				     DBUS_TYPE_INVALID);
+
+	helper = g_slice_new (GetUnreadMessagesHelper);
+	helper->unread_msgs_count = unread_msgs_count_v;
+	dbus_message_ref (message);
+	helper->message = message;
+	helper->con = con;
+	helper->account_hits_list = NULL;
+	helper->inboxes_list = NULL;
+	helper->accounts_list = TNY_LIST (tny_simple_list_new ());
+	helper->mail_op = modest_mail_operation_new (NULL);
+	modest_mail_operation_queue_add (modest_runtime_get_mail_operation_queue (),
+					 helper->mail_op);
+
+	g_idle_add ((GSourceFunc) on_idle_get_unread_messages, helper);
+}
+
 
 /* A complex D-Bus type (like a struct),
  * used to return various information about a folder.
@@ -2186,6 +2863,22 @@ modest_dbus_req_filter (DBusConnection *con,
 			handled = TRUE;
 		}
 			 	
+	} else if (dbus_message_is_method_call (message,
+					 MODEST_DBUS_IFACE,
+					 MODEST_DBUS_METHOD_GET_UNREAD_MESSAGES)) {
+		
+	/* don't try to get unread messages when there not enough mem */
+		if (modest_platform_check_memory_low (NULL, TRUE)) {
+			g_warning ("%s: not enough memory for searching",
+				   __FUNCTION__);
+			reply_empty_results (con, message);
+			handled = TRUE;
+
+		} else {
+			on_dbus_method_get_unread_messages (con, message);
+			handled = TRUE;
+		}
+			 	
 	} else if (dbus_message_is_method_call (message,
 						MODEST_DBUS_IFACE,
 						MODEST_DBUS_METHOD_GET_FOLDERS)) {
@@ -2237,3 +2930,115 @@ notify_error_in_dbus_callback (gpointer user_data)
 
 	return FALSE;
 }
+
+gboolean
+modest_dbus_emit_account_created_signal (DBusConnection *con, 
+					 const gchar *account_id)
+{
+	DBusMessage *signal;
+	dbus_bool_t result;
+
+	signal = dbus_message_new_signal (MODEST_DBUS_OBJECT, MODEST_DBUS_IFACE,
+					  MODEST_DBUS_SIGNAL_ACCOUNT_CREATED);
+
+	if (signal == NULL)
+		return FALSE;
+
+	if (!dbus_message_append_args  (signal,
+					DBUS_TYPE_STRING, &account_id,
+					DBUS_TYPE_INVALID)) {
+		dbus_message_unref (signal);
+		return FALSE;
+	}
+
+	result = dbus_connection_send (con, signal, NULL);
+
+	dbus_message_unref (signal);
+
+	return result?TRUE:FALSE;
+}
+
+gboolean
+modest_dbus_emit_account_removed_signal (DBusConnection *con,
+					 const gchar *account_id)
+{
+	DBusMessage *signal;
+	dbus_bool_t result;
+
+	signal = dbus_message_new_signal (MODEST_DBUS_OBJECT, MODEST_DBUS_IFACE,
+					  MODEST_DBUS_SIGNAL_ACCOUNT_REMOVED);
+
+	if (signal == NULL)
+		return FALSE;
+
+	if (!dbus_message_append_args  (signal,
+					DBUS_TYPE_STRING, &account_id,
+					DBUS_TYPE_INVALID)) {
+		dbus_message_unref (signal);
+		return FALSE;
+	}
+
+	result = dbus_connection_send (con, signal, NULL);
+
+	dbus_message_unref (signal);
+
+	return result?TRUE:FALSE;
+}
+
+gboolean
+modest_dbus_emit_folder_updated_signal (DBusConnection *con, 
+					const gchar *account_id,
+					const gchar *folder_path)
+{
+	DBusMessage *signal;
+	dbus_bool_t result;
+
+	signal = dbus_message_new_signal (MODEST_DBUS_OBJECT, MODEST_DBUS_IFACE,
+					  MODEST_DBUS_SIGNAL_FOLDER_UPDATED);
+
+	if (signal == NULL)
+		return FALSE;
+
+	if (!dbus_message_append_args  (signal,
+					DBUS_TYPE_STRING, &account_id,
+					DBUS_TYPE_STRING, &folder_path,
+					DBUS_TYPE_INVALID)) {
+		dbus_message_unref (signal);
+		return FALSE;
+	}
+
+	result = dbus_connection_send (con, signal, NULL);
+
+	dbus_message_unref (signal);
+
+	return result?TRUE:FALSE;
+}
+
+gboolean
+modest_dbus_emit_msg_read_changed_signal (DBusConnection *con,
+					  const gchar *uid,
+					  gboolean is_read)
+{
+	DBusMessage *signal;
+	dbus_bool_t result;
+
+	signal = dbus_message_new_signal (MODEST_DBUS_OBJECT, MODEST_DBUS_IFACE,
+					  MODEST_DBUS_SIGNAL_MSG_READ_CHANGED);
+
+	if (signal == NULL)
+		return FALSE;
+
+	if (!dbus_message_append_args  (signal,
+					DBUS_TYPE_STRING, &uid,
+					DBUS_TYPE_BOOLEAN, &is_read,
+					DBUS_TYPE_INVALID)) {
+		dbus_message_unref (signal);
+		return FALSE;
+	}
+
+	result = dbus_connection_send (con, signal, NULL);
+
+	dbus_message_unref (signal);
+
+	return result?TRUE:FALSE;
+}
