@@ -471,7 +471,7 @@ modest_platform_connect_and_perform_if_network_folderstore (GtkWindow *parent_wi
 
 
 void 
-modest_platform_remove_new_mail_notifications (gboolean only_visuals)
+modest_platform_remove_new_mail_notifications (gboolean only_visuals, const gchar *acc_name)
 {
 	g_warning ("NOT IMPLEMENTED %s", __FUNCTION__);
 }
@@ -652,3 +652,35 @@ modest_platform_get_list_to_move (ModestWindow *window)
 
 	return list;
 }
+
+DBusConnection*
+modest_platform_get_dbus_connection (void)
+{
+	return NULL;
+}
+
+void
+modest_platform_emit_folder_updated_signal (const gchar *account_id, const gchar *folder_id)
+{
+	return;
+}
+
+
+void
+modest_platform_emit_account_created_signal (const gchar *account_id)
+{
+	return;
+}
+
+void
+modest_platform_emit_account_removed_signal (const gchar *account_id)
+{
+	return;
+}
+
+void
+modest_platform_emit_msg_read_changed_signal (const gchar *msg_uid,
+					      gboolean is_read)
+{
+	return;
+}
