@@ -62,16 +62,22 @@ extern const guint USER_NAME_FORBIDDEN_CHARS_LENGTH;
 /**
  * modest_text_utils_derived_subject:
  * @subject: a string which contains the original subject
- * @prefix: the prefix for the new subject (such as 'Re:' or 'Fwd:'),
- *           must not be NULL
+ * @is_reply: whether the derived subject is for a reply or a forward message
+ *
+ * create a 'derived' subject line for eg. replies and forwards. Note
+ * that this function will use the localized versions of "Re" and
+ * "Fw", unless one of these two versions was already included. For
+ * example replying to an email in Finish would work as:
+ *
+ * "some subject"     -> "VS: some subject"
+ * "VS: some subject" -> "VS: some subject"
+ * "Re: some subject" -> "Re: some subject"
+ * "Fw: some subject" -> "VS: Fw: some subject"
  *
- * create a 'derived' subject line for eg. replies and forwards 
- * 
  * Returns: a newly allocated string containing the resulting subject
- * subject == NULL, then @prefix " " will be returned
  */
-gchar* modest_text_utils_derived_subject (const gchar *subject, 
-					  const gchar* prefix);
+gchar* modest_text_utils_derived_subject (const gchar *subject,
+					  gboolean is_reply);
 
 
 /**
@@ -561,4 +567,10 @@ gchar * modest_text_utils_get_secure_header (const gchar *value, const gchar *he
  **/
 gchar * modest_text_utils_quote_names (const gchar *recipients);
 
+gboolean modest_text_utils_no_recipient (GtkTextBuffer *buffer);
+
+gchar * modest_text_utils_create_colored_signature (const gchar *signature);
+
+gboolean modest_text_utils_live_search_find (const gchar *haystack, const gchar *needles);
+
 #endif /* __MODEST_TEXT_UTILS_H__ */
